'use strict';

import webdriver from 'selenium-webdriver';
import { times } from 'lodash';
import { DriverUtilities } from './DriverUtilities';
import { SimpleLogger } from './SimpleLogger';
const Eyes = require('eyes.selenium').Eyes;

// What config should look like at least:
const ConfigStructure = {
  defaultTimeout: 10000,
  defaultPageLoadTimeout: 10000,
  applicationName: "used for logging",
  baseUrl: "http://somesite.com",
  seleniumGrid: {
    host: "",
    username: "",
    accessKey: "",
  },
  eyesConfig: {
    apiKey: "",
  },
  capabilities: { // specific things that need to be provided to BrowserStack/SauceLabs/SeGrid/etc.
    // Other properties that help decide what browser to use like browserName, platform, version, etc.
    // Also the API key usually needs to be in here somewhere (check specific documentation for cloud provider).
  },
  scrollBuffer: 25, // the amount of pixels to scroll down (or up if negative) after a scrollTo, to account for floaties and errors
};

export default class DriverContainer {
  initialize(local){
    this.by = webdriver.By;
    this.eyesOpened = false;
    
    let builder = new webdriver.Builder();
    // http://seleniumhq.github.io/selenium/docs/api/javascript/module/selenium-webdriver/index_exports_Builder.html
    if (local) {
      this.driver = builder.forBrowser(this.config.capabilities.browserName).build();
    } else {
      this.driver = builder.usingServer(this.config.seleniumGrid.host).withCapabilities(this.config.capabilities).build();
    }
    
    let _this = this;
  
    return this.driver.getSession().then(function (sessionId) {
      _this.driver.allPassed = true;
      _this.driver.sessionID = sessionId.id_;
      _this.driver.manage().timeouts().implicitlyWait(_this.config.defaultTimeout);
      _this.driver.manage().timeouts().pageLoadTimeout(_this.config.defaultPageLoadTimeout);
      _this.driverUtilities = new DriverUtilities(_this.config, _this.logger, _this.driver, _this.defaultTimeout);
    });
  }
  
  openEyes(TestName, batchName, batchId) {
    // set up applitools eyes
    var _this = this;
    _this.eyes = new Eyes();
    _this.eyes.setApiKey(this.config.eyesConfig.apiKey);
    _this.eyes.setForceFullPageScreenshot(true);
    _this.eyes.setStitchMode(Eyes.StitchMode.CSS);
    _this.eyes.setBatch(batchName, batchId);
    _this.eyes.setDefaultMatchTimeout(3000); // TODO refine this stuff. Do we need to go more/less for the carosels? Can we provide this after opening eyes?
  
    return _this.eyes.open(
      _this.driver,
      _this.config.applicationName,
      TestName
    ).then(function(driver) {
      _this.driver = driver;
      _this.eyesOpened = true;
    });
  }
  
  closeEyes() {
    var _this = this;
    return _this.quitBrowser().then(function(){
      return _this.eyes.close(false);
    }).then(function(){
      _this.eyesOpened = false;
    });
  }
  
  forceCloseEyes(){
    if (this.eyesOpened) {
      this.eyes.abortIfNotClosed();
    }
  }
  
  constructor(config, logger) {
    this.logger = logger;
    if (!this.logger) {
      this.logger = new SimpleLogger();
    }
    
    this.config = config;
    
    // set run name
    this.testName = this.config.applicationName;
    this.logger.runName = this.testName;
    if (!this.config.capabilities.name) this.config.capabilities.name = this.testName;
  }
  
  clearCookiesAndDeleteLocalStorage() {
    return this.clearCookies().then(function(){
      return this.deleteLocalStorage();
    });
  }
  
  clearCookies() {
    return this.driver.manage().deleteAllCookies();
  }
  
  deleteLocalStorage() {
    return this.driver.getCurrentUrl().then((url) => {
      if (url.startsWith('data:') === false && url !== 'about:blank') {
        return this.driver.executeScript('window.localStorage.clear();');
      }
    });
  }
  
  quitBrowser() {
    return this.driver.quit();
  }
  
  // TODO From the WP-e2e-tests, not sure if this can easily be cleaner, or modular to dismiss all alerts instead of default 3, maybe accept -1 as all?
  dismissAlerts(qty = 3) {
    times(qty, () => {
      return this.driver.get('data:,').then(() => {
      }, (err) => {
        return this.driver.sleep(2000).then(() => {
          this.logger.warn(`Accepting an open alert: '${err}'`);
          return this.driver.switchTo().alert().then((alert) => {
            return alert.accept();
          }, () => {
          });
        });
      });
    });
  }
  
}
